package com.demo.simulations;

import com.demo.utils.ApplicationEndPoints;
import com.demo.utils.ConfigReader;
import com.demo.utils.FeederUtils;
import com.demo.utils.UserDataProvider;
import io.gatling.core.scenario.Scenario;
import io.gatling.javaapi.core.FeederBuilder;
import io.gatling.javaapi.core.ScenarioBuilder;
import io.gatling.javaapi.core.Simulation;
import io.gatling.javaapi.http.HttpRequestActionBuilder;

import java.io.IOException;
import java.util.Map;

import static io.gatling.javaapi.core.CoreDsl.*;
import static io.gatling.javaapi.http.HttpDsl.http;
import static io.gatling.javaapi.http.HttpDsl.status;

public class LoginPageSimulation extends Simulation {

    ScenarioBuilder loginPageScenario = scenario("Login Page Load Test Scenario")
            .exec(session -> {
                // Get random user and inject into session
                Map<String, String> user = UserDataProvider.getRandomUser();
                return session
                        .set("username", user.get("username"))
                        .set("password", user.get("password"));
            })
            .exec(http("Submit Login Request")
                    .post(ApplicationEndPoints.Login_END_POINT)
                    .header("Content-Type", "application/json")
                    .body(StringBody(
                            "{ \"username\": \"#{username}\", \"password\": \"#{password}\" }"
                    )).asJson()
                    .check(status().is(200))
                    // .check(jsonPath("$.accessToken").saveAs("accessToken"))
                    .check(bodyString().saveAs("responseBody"))
            )
            .exec(session -> {
                System.out.println("Response Body: " + session.getString("responseBody"));
                return session;
            });


    /*
      .exec(http("Login Page Request")
              .get(ApplicationEndPoints.Login_END_POINT)
              .check(status().is(200))
              .check(css("input[name=_csrf]", "value").saveAs("csrfToken"))
      )
      .pause(2)
      .exec(http("Submit Login Form")
              .post(ApplicationEndPoints.Login_END_POINT)
              .formParam("username", "#{username}")
              .formParam("password", "#{password}")
              .formParam("_csrf", "#{csrfToken}")
              .check(status().is(302)) // Expect redirect
      );




     */ {
        try {
            setUp(
                    loginPageScenario.injectOpen(
                            rampUsers(new ConfigReader().getRampUPCount())
                                    .during(new ConfigReader().getRampupDuration())
                    ).protocols(new BaseSimulation().baseInitialization())
            );
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }

}





    /*

     FeederBuilder.Batchable<String> csvFeeder = FeederUtils.getCSVFeeder("users.csv");

    ScenarioBuilder loginPageScenario = scenario("Login Page Load Test Scenario")
            .feed(new ) // 1️⃣ Read username/password from CSV
            .exec(http("Login Page Request")
                    .get(ApplicationEndPoints.Login_END_POINT)
                    .check(status().is(200))
                    .check(css("input[name=_csrf]", "value").saveAs("csrfToken")) // 2️⃣ Capture CSRF token
            )
            .pause(2) // simulate think time
            .exec(http("Submit Login Form")
                    .post(ApplicationEndPoints.Login_END_POINT)
                    .formParam("username", "#{username}")   // 3️⃣ Read from CSV
                    .formParam("password", "#{password}")   // 3️⃣ Read from CSV
                    .formParam("_csrf", "#{csrfToken}")     // 4️⃣ Use captured CSRF token
                    .check(status().is(302)) // 5️⃣ Expect redirect after successful login
            );

    {
        try {
            setUp(
                    loginPageScenario.injectOpen(
                            rampUsers(new ConfigReader().getRampUPCount())
                                    .during(new ConfigReader().getRampupDuration())
                    ).protocols(new BaseSimulation().baseInitialization())
            );
        } catch (IOException e) {
            throw new RuntimeException(e);
        }
    }


    }
    }
     */
